/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2006 Dr. Lars H. Hahn

*/

package haushalt.daten;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.Locale;

/**
 * Reprsentiert einen Geldbetrag.
 * Die Whrung des Betrags ist in der Regel Euro, das Whrungssymbol kann aber
 * gendert werden.
 * @author Dr. Lars H. Hahn
 * @version 2.1/2006.02.02
 */

/*
 * 2006.02.02 BugFix: Plus-Zeichen beim Pharsen ignorieren
 */

public class Euro implements Cloneable, Comparable<Euro> {


  private static final NumberFormat nf = NumberFormat.getInstance(Locale.GERMAN);
  public static final Euro NULL_EURO = new Euro();
	private static String symbol = "";
  private long wert = 0L;

  public Euro() {
    nf.setMinimumFractionDigits(2);
    nf.setMaximumFractionDigits(2);
  }

  public Euro(double wert) {
    nf.setMinimumFractionDigits(2);
    nf.setMaximumFractionDigits(2);
    setWert(wert);
  }

  public Euro(String wert) {
    if (wert.startsWith("+")) {
       wert = wert.substring(1); 
    }
    if(wert != "") {
      try {
        if(wert.endsWith(symbol))
          setWert(nf.parse(wert.substring(0, wert.length()-symbol.length())).doubleValue());
        else
          setWert(nf.parse(wert).doubleValue());
      }
      catch(java.text.ParseException e) {
				this.wert = 0L;
      }
    }
    else
  		this.wert = 0L;
  }

	public String toString() {
		return nf.format(wert / 100.0D) + " " + symbol;
	}

	public double toDouble() {
		return wert / 100.0D;
	}

  private void setWert(double wert) {
    double help = wert * 100.0D;
    if(help < 0)
      this.wert = (long) (help - 0.5D);
    else
      this.wert = (long) (help + 0.5D);
  }

  public static void setWaehrungssymbol(String waehrungssymbol) {
    symbol = waehrungssymbol;
  }

  public void umrechnenVonDM() { // Wenn der Wert in DM vorliegt -> umrechnen in EURO
    if(wert < 0)
      wert = (long)(wert / 1.95583D - 0.5D);
    else
      wert = (long)(wert / 1.95583D + 0.5D);
  }

  public boolean equals(Object obj) {
    if(obj == null)
      return false;
    if(obj.getClass() != Euro.class)
      return false;
    return ((Euro)obj).wert == wert;
  }

  public int hashCode() {
    assert false : "hashCode not designed";
    return 0;
  }
  
  /**
   * Bildet die Summe aus diesem Euro-Objekt und dem Parameter.
   * Die Summe wird einem neuen Objekt zu gewiesen. Um aufzusummieren sollte
   * deshalb die Funktion <code>sum(Euro)</code> verwendet werden.
   * @see Euro#sum(Euro)
   * @param euro
   * @return Neues Objekt mit der Summe
   */
  public Euro add(Euro euro) {
    Euro ergebnis = new Euro();
    ergebnis.wert = this.wert + euro.wert;
    return ergebnis;
  }

  public Euro sub(Euro euro) {
    Euro ergebnis = new Euro();
    ergebnis.wert = this.wert - euro.wert;
    return ergebnis;
  }

  public Euro durch(int zahl) {
    Euro ergebnis = new Euro();
    ergebnis.wert = this.wert / zahl;
    return ergebnis;
  }
  
  public Euro mal(double zahl) {
		Euro ergebnis = new Euro();
		ergebnis.setWert(toDouble() * zahl);
		return ergebnis;
  }
  
  public void sum(Euro euro) {
    wert += euro.wert;
  }

	// -- E/A-Funktionen -------------------------------------------------------

	public void laden(DataInputStream in)
		throws IOException {
		wert = in.readLong();
	}

	public void speichern(DataOutputStream out)
		throws IOException {
		out.writeLong(wert);
	}

	// -- Methoden fuer Interface: Cloneable --------------------

	final public Object clone() {
		Euro kopie = new Euro();
		kopie.wert = this.wert;
		return kopie;
	}

  // -- Methoden fuer Interface: Comparable -------------------

  public int compareTo(Euro euro) {
    if(wert < euro.wert)
      return -1;
    if(wert > euro.wert)
      return 1;
    return 0;
  }

}