/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2006 Dr. Lars H. Hahn

*/

package haushalt.gui.dialoge;

import haushalt.daten.Datenbasis;
import haushalt.daten.Euro;
import haushalt.daten.EinzelKategorie;
import haushalt.daten.Kategorie;
import haushalt.gui.DeleteableTextField;
import haushalt.gui.EuroRenderer;
import haushalt.gui.Haushalt;
import haushalt.gui.KategorieRenderer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;

/**
 * Ermglicht das nachtrgliche ndern der Kategorien von 
 * Buchungen unabhngig von den Registern
 * 
 * @author Dr. Lars H. Hahn
 * @version 2.1/2006.02.10
 */

/*
 * 2006.02.10 Erste Version
 */

public class DlgBereinigen extends JDialog {
  private static final long serialVersionUID = 1L;

  private final JComboBox comboBox = new JComboBox();
  private final DefaultListModel listModel = new DefaultListModel();
  private final JList list = new JList(listModel);
  private final JCheckBox checkBox = new JCheckBox("Unterkategorien verwenden", true);
  private final Datenbasis db;
  
  public DlgBereinigen(final Haushalt haushalt, final Datenbasis db) {
    super(haushalt.getFrame(), "Kategorien bereinigen", true);
    this.db = db;
    final JPanel northPane = new JPanel();
    final JPanel southPane = new JPanel();
    final JPanel eastPane = new JPanel();
    final BereinigenTableModel tableModel = new BereinigenTableModel(db);
    final TableSorter sorter = new TableSorter(tableModel);
    final JTable table = new JTable(sorter);
    sorter.setTableHeader(table.getTableHeader());
    final JLabel label1 = new JLabel("Kategorie:");
    final JLabel label2 = new JLabel("Buchungstext:");
    comboBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(comboBox.getSelectedIndex() == 0)
          tableModel.setKategorie(null);
        else
          tableModel.setKategorie((EinzelKategorie) comboBox.getSelectedItem());
      }
    });
    final DeleteableTextField buchungstext = new DeleteableTextField(20);
    buchungstext.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        tableModel.setBuchungstext(buchungstext.getText());
      }
    });
    table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    table.setDefaultRenderer(Kategorie.class, new KategorieRenderer());
    table.setDefaultRenderer(Euro.class, new EuroRenderer());
    checkBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e)  {
        kategorienEinlesen();
        tableModel.setUnterkategorien(checkBox.isSelected());
      }
    });
    final JButton buttonZuordnen = new JButton("Kategorie neu zuweisen");
    buttonZuordnen.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(table.getSelectedRowCount() == 0) {
          JOptionPane.showMessageDialog(haushalt.getFrame(),
            "Es sind keine Buchungen selektiert.");
          return;
        }
        if(list.getSelectedIndex() == -1) {
          JOptionPane.showMessageDialog(haushalt.getFrame(),
            "Es sind keine neue Kategorie selektiert.");
          return;
        }
        EinzelKategorie neueKategorie = (EinzelKategorie) list.getSelectedValue();
        int[] selectedRows = table.getSelectedRows();
        int[] modelRows = new int[selectedRows.length];
        for (int i = 0; i < modelRows.length; i++)
          modelRows[i] = sorter.modelIndex(selectedRows[i]);
        tableModel.setNeueKategorie(modelRows, neueKategorie);
      }
    });
    final JButton buttonBearbeiten = new JButton("Kategorien bearbeiten", haushalt.bildLaden("Auto16.gif"));
    buttonBearbeiten.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        DlgKategorienBearbeiten dlg = new DlgKategorienBearbeiten(haushalt, db);
        dlg.showDialog();
        kategorienEinlesen();
      }
    });
    final JButton buttonAbbruch = new JButton("Schlieen");
    buttonAbbruch.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    });
    final JScrollPane scrollPane = new JScrollPane(list);
    scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    northPane.setBorder(BorderFactory.createTitledBorder("Auswahlfilter"));
    northPane.setLayout(new BoxLayout(northPane, BoxLayout.LINE_AXIS));
    northPane.add(label1);
    northPane.add(Box.createRigidArea(new Dimension(5,0)));
    northPane.add(comboBox);
    northPane.add(Box.createRigidArea(new Dimension(10,0)));
    northPane.add(label2);
    northPane.add(Box.createRigidArea(new Dimension(5,0)));
    northPane.add(buchungstext);
    southPane.add(buttonAbbruch);
    southPane.add(buttonBearbeiten);
    southPane.add(buttonZuordnen);
    eastPane.add(scrollPane);
    eastPane.add(checkBox);
    eastPane.setBorder(BorderFactory.createTitledBorder("Neue Kategorie"));
    eastPane.setLayout(new BoxLayout(eastPane, BoxLayout.Y_AXIS));
    getContentPane().add(northPane, BorderLayout.NORTH);
    getContentPane().add(new JScrollPane(table), BorderLayout.CENTER);
    getContentPane().add(southPane, BorderLayout.SOUTH);
    getContentPane().add(eastPane, BorderLayout.EAST);
    kategorienEinlesen();
  }
  
  private void kategorienEinlesen() {
    Object selectedItem = comboBox.getSelectedItem();
    comboBox.removeAllItems();
    listModel.removeAllElements();
    EinzelKategorie[] kategorien = db.getKategorien(checkBox.isSelected());
    comboBox.addItem("<beliebige Kategorie>");
    for (int i = 0; i < kategorien.length; i++) {
      comboBox.addItem(kategorien[i]);
      listModel.addElement(kategorien[i]);
    }
    if(selectedItem == null)
      comboBox.setSelectedItem(EinzelKategorie.SONSTIGES);
    else
      comboBox.setSelectedItem(selectedItem);
  }

}
